@@ -807,11 +807,9 @@ uint32_t acp_card_find_best_profile_index(struct acp_card *card, const char *nam
 		struct acp_card_profile *p = profiles[i];
 
 		if (name) {
-			if (strcmp(name, p->name))
+			if (strcmp(name, p->name) == 0)
 				best = i;
-			continue;
-		}
-		if (p->flags & ACP_PROFILE_OFF) {
+		} else if (p->flags & ACP_PROFILE_OFF) {
 			off = i;
 		} else if (p->available == ACP_AVAILABLE_YES) {
 			if (best == ACP_INVALID_INDEX || p->priority > profiles[best]->priority)
@@ -1179,11 +1177,16 @@ static int device_enable(pa_card *impl, pa_alsa_mapping *mapping, pa_alsa_device
 		p->port.priority = p->priority;
 	}
 
-	port_index = acp_device_find_best_port_index(&dev->device, NULL);
+	if (impl->auto_port)
+		port_index = acp_device_find_best_port_index(&dev->device, NULL);
+	else
+		port_index = ACP_INVALID_INDEX;
+
 	if (port_index == ACP_INVALID_INDEX)
 		dev->active_port = NULL;
 	else
 		dev->active_port = (pa_device_port*)impl->card.ports[port_index];
+
 	if (dev->active_port)
 		dev->active_port->port.flags |= ACP_PORT_ACTIVE;
 
@@ -1346,6 +1349,8 @@ struct acp_card *acp_card_new(uint32_t index, const struct acp_dict *props)
 	card->active_profile_index = ACP_INVALID_INDEX;
 
 	impl->use_ucm = true;
+	impl->auto_profile = true;
+	impl->auto_port = true;
 
 	if (props) {
 		if ((s = acp_dict_lookup(props, "api.alsa.use-ucm")) != NULL)
@@ -1358,6 +1363,10 @@ struct acp_card *acp_card_new(uint32_t index, const struct acp_dict *props)
 			profile_set = s;
 		if ((s = acp_dict_lookup(props, "device.profile")) != NULL)
 			profile = s;
+		if ((s = acp_dict_lookup(props, "api.acp.auto-profile")) != NULL)
+			impl->auto_profile = (strcmp(s, "true") == 0 || atoi(s) == 1);
+		if ((s = acp_dict_lookup(props, "api.acp.auto-port")) != NULL)
+			impl->auto_port = (strcmp(s, "true") == 0 || atoi(s) == 1);
 	}
 
 	impl->ucm.default_sample_spec.format = PA_SAMPLE_S16NE;
@@ -1425,6 +1434,9 @@ struct acp_card *acp_card_new(uint32_t index, const struct acp_dict *props)
 
 	init_jacks(impl);
 
+	if (!impl->auto_profile && profile == NULL)
+		profile = "off";
+
 	profile_index = acp_card_find_best_profile_index(&impl->card, profile);
 	acp_card_set_profile(&impl->card, profile_index);
 
@@ -1579,11 +1591,9 @@ uint32_t acp_device_find_best_port_index(struct acp_device *dev, const char *nam
 		struct acp_port *p = ports[i];
 
 		if (name) {
-			if (strcmp(name, p->name))
+			if (strcmp(name, p->name) == 0)
 				best = i;
-			continue;
-		}
-		if (p->available == ACP_AVAILABLE_YES) {
+		} else if (p->available == ACP_AVAILABLE_YES) {
 			if (best == ACP_INVALID_INDEX || p->priority > ports[best]->priority)
 				best = i;
 		} else if (p->available != ACP_AVAILABLE_NO) {
