@@ -46,6 +46,7 @@
 #include <spa/utils/keys.h>
 #include <spa/utils/names.h>
 #include <spa/utils/result.h>
+#include <spa/utils/json.h>
 
 #include "a2dp-codecs.h"
 #include "defs.h"
@@ -68,8 +69,13 @@ struct spa_bt_monitor {
 	uint32_t count;
 	uint32_t id;
 
+	/*
+	 * Lists of BlueZ objects, kept up-to-date by following DBus events
+	 * initiated by BlueZ. Object lifetime is also determined by that.
+	 */
 	struct spa_list adapter_list;
 	struct spa_list device_list;
+	struct spa_list remote_endpoint_list;
 	struct spa_list transport_list;
 
 	unsigned int filters_added:1;
@@ -79,9 +85,54 @@ struct spa_bt_monitor {
 	struct spa_bt_backend *backend_ofono;
 	struct spa_bt_backend *backend_hsphfpd;
 
+	struct spa_dict enabled_codecs;
+
 	unsigned int enable_sbc_xq:1;
 };
 
+/* Stream endpoints owned by BlueZ for each device */
+struct spa_bt_remote_endpoint {
+	struct spa_list link;
+	struct spa_list device_link;
+	struct spa_bt_monitor *monitor;
+	char *path;
+
+	char *uuid;
+	unsigned int codec;
+	struct spa_bt_device *device;
+	uint8_t *capabilities;
+	int capabilities_len;
+	bool delay_reporting;
+};
+
+/*
+ * Codec switching tries various codec/remote endpoint combinations
+ * in order, until an acceptable one is found. This triggers BlueZ
+ * to initiate DBus calls that result to the creation of a transport
+ * with the desired capabilities.
+ * The codec switch struct tracks candidates still to be tried.
+ */
+struct spa_bt_a2dp_codec_switch {
+	struct spa_bt_device *device;
+	struct spa_list device_link;
+
+	DBusPendingCall *pending;
+
+	uint32_t profile;
+
+	/*
+	 * Called asynchronously, so endpoint paths instead of pointers (which may be
+	 * invalidated in the meantime).
+	 */
+	const struct a2dp_codec **codecs;
+	char **paths;
+
+	const struct a2dp_codec **codec_iter;	/**< outer iterator over codecs */
+	char **path_iter;			/**< inner iterator over endpoint paths */
+
+	size_t num_paths;
+};
+
 /*
  * SCO socket connect may fail with ECONNABORTED if it is done too soon after
  * previous close. To avoid this in cases where nodes are toggled between
@@ -132,6 +183,11 @@ static const struct a2dp_codec *a2dp_endpoint_to_codec(const char *endpoint)
 	return NULL;
 }
 
+static bool is_a2dp_codec_enabled(struct spa_bt_monitor *monitor, const struct a2dp_codec *codec)
+{
+	return spa_dict_lookup(&monitor->enabled_codecs, codec->name) != NULL;
+}
+
 static DBusHandlerResult endpoint_select_configuration(DBusConnection *conn, DBusMessage *m, void *userdata)
 {
 	struct spa_bt_monitor *monitor = userdata;
@@ -158,13 +214,8 @@ static DBusHandlerResult endpoint_select_configuration(DBusConnection *conn, DBu
 		spa_log_debug(monitor->log, "  %d: %02x", i, cap[i]);
 
 	codec = a2dp_endpoint_to_codec(path);
-	if (codec != NULL) {
-		struct spa_dict_item items[1];
-
-		items[0] = SPA_DICT_ITEM_INIT("codec.sbc.enable-xq", monitor->enable_sbc_xq ? "true" : "false");
-
-		res = codec->select_config(codec, 0, cap, size, &SPA_DICT_INIT_ARRAY(items), config);
-	}
+	if (codec != NULL)
+		res = codec->select_config(codec, 0, cap, size, NULL, config);
 	else
 		res = -ENOTSUP;
 
@@ -363,7 +414,11 @@ static struct spa_bt_device *device_create(struct spa_bt_monitor *monitor, const
 	d->id = monitor->id++;
 	d->monitor = monitor;
 	d->path = strdup(path);
+	spa_list_init(&d->remote_endpoint_list);
 	spa_list_init(&d->transport_list);
+	spa_list_init(&d->codec_switch_list);
+
+	spa_hook_list_init(&d->listener_list);
 
 	spa_list_prepend(&monitor->device_list, &d->link);
 
@@ -372,20 +427,38 @@ static struct spa_bt_device *device_create(struct spa_bt_monitor *monitor, const
 
 static int device_stop_timer(struct spa_bt_device *device);
 
+static int device_remove(struct spa_bt_monitor *monitor, struct spa_bt_device *device);
+
+static void a2dp_codec_switch_free(struct spa_bt_a2dp_codec_switch *sw);
+
 static void device_free(struct spa_bt_device *device)
 {
-	struct spa_bt_transport *t;
+	struct spa_bt_remote_endpoint *ep, *tep;
+	struct spa_bt_a2dp_codec_switch *sw;
+	struct spa_bt_transport *t, *tt;
 	struct spa_bt_monitor *monitor = device->monitor;
 
 	spa_log_debug(monitor->log, "%p", device);
 	device_stop_timer(device);
+	device_remove(monitor, device);
 
-	spa_list_for_each(t, &device->transport_list, device_link) {
+	spa_list_for_each_safe(ep, tep, &device->remote_endpoint_list, device_link) {
+		if (ep->device == device) {
+			spa_list_remove(&ep->device_link);
+			ep->device = NULL;
+		}
+	}
+
+	spa_list_for_each_safe(t, tt, &device->transport_list, device_link) {
 		if (t->device == device) {
 			spa_list_remove(&t->device_link);
 			t->device = NULL;
 		}
 	}
+
+	spa_list_consume(sw, &device->codec_switch_list, device_link)
+		a2dp_codec_switch_free(sw);
+
 	spa_list_remove(&device->link);
 	free(device->path);
 	free(device->alias);
@@ -523,7 +596,7 @@ int spa_bt_device_check_profiles(struct spa_bt_device *device, bool force)
 	spa_log_debug(monitor->log, "device %p: profiles %08x %08x %d",
 			device, device->profiles, connected_profiles, device->added);
 
-	if (connected_profiles == 0) {
+	if (connected_profiles == 0 && device->active_codec_switch == NULL) {
 		if (device->added) {
 			device_stop_timer(device);
 			device_remove(monitor, device);
@@ -553,8 +626,11 @@ static void device_set_connected(struct spa_bt_device *device, int connected)
 
 int spa_bt_device_connect_profile(struct spa_bt_device *device, enum spa_bt_profile profile)
 {
+	uint32_t prev_connected = device->connected_profiles;
 	device->connected_profiles |= profile;
 	spa_bt_device_check_profiles(device, false);
+	if (device->connected_profiles != prev_connected)
+		spa_bt_device_emit_profiles_changed(device, device->profiles, prev_connected);
 	return 0;
 }
 
@@ -666,6 +742,7 @@ static int device_update_props(struct spa_bt_device *device,
 		}
 		else if (strcmp(key, "UUIDs") == 0) {
 			DBusMessageIter iter;
+			uint32_t prev_profiles = device->profiles;
 
 			if (!check_iter_signature(&it[1], "as"))
 				goto next;
@@ -685,6 +762,10 @@ static int device_update_props(struct spa_bt_device *device,
 				}
 				dbus_message_iter_next(&iter);
 			}
+
+			if (device->profiles != prev_profiles)
+				spa_bt_device_emit_profiles_changed(
+					device, prev_profiles, device->connected_profiles);
 		}
 		else
 			spa_log_debug(monitor->log, "device %p: unhandled key %s type %d", device, key, type);
@@ -695,6 +776,79 @@ static int device_update_props(struct spa_bt_device *device,
 	return 0;
 }
 
+static bool device_can_accept_a2dp_codec(struct spa_bt_device *device, const struct a2dp_codec *codec)
+{
+	struct spa_bt_monitor *monitor = device->monitor;
+
+	/* Device capability checks in addition to A2DP Capabilities. */
+
+	if (!is_a2dp_codec_enabled(device->monitor, codec))
+		return false;
+
+	if (codec->feature_flag != NULL && strcmp(codec->feature_flag, "sbc-xq") == 0)
+		return monitor->enable_sbc_xq;
+
+	/* The rest is determined by A2DP caps */
+	return true;
+}
+
+bool spa_bt_device_supports_a2dp_codec(struct spa_bt_device *device, const struct a2dp_codec *codec)
+{
+	struct spa_bt_remote_endpoint *ep;
+
+	if (!device_can_accept_a2dp_codec(device, codec))
+		return false;
+
+	if (!device->adapter->application_registered) {
+		/* Codec switching not supported: only plain SBC allowed */
+		return (codec->codec_id == A2DP_CODEC_SBC && strcmp(codec->name, "sbc") == 0);
+	}
+
+	spa_list_for_each(ep, &device->remote_endpoint_list, device_link) {
+		if (a2dp_codec_check_caps(codec, ep->codec, ep->capabilities, ep->capabilities_len))
+			return true;
+	}
+
+	return false;
+}
+
+const struct a2dp_codec **spa_bt_device_get_supported_a2dp_codecs(struct spa_bt_device *device, size_t *count)
+{
+	const struct a2dp_codec **supported_codecs;
+	size_t i, j, size;
+
+	*count = 0;
+
+	size = 8;
+	supported_codecs = malloc(size * sizeof(const struct a2dp_codec *));
+	if (supported_codecs == NULL)
+		return NULL;
+
+	j = 0;
+	for (i = 0; a2dp_codecs[i] != NULL; ++i) {
+		if (spa_bt_device_supports_a2dp_codec(device, a2dp_codecs[i])) {
+			supported_codecs[j] = a2dp_codecs[i];
+			++j;
+		}
+
+		if (j >= size) {
+			const struct a2dp_codec **p;
+			size = size * 2;
+			p = realloc(supported_codecs, size * sizeof(const struct a2dp_codec *));
+			if (p == NULL) {
+				free(supported_codecs);
+				return NULL;
+			}
+			supported_codecs = p;
+		}
+	}
+
+	supported_codecs[j] = NULL;
+	*count = j;
+
+	return supported_codecs;
+}
+
 static int device_try_connect_profile(struct spa_bt_device *device,
                                       enum spa_bt_profile profile,
                                       const char *profile_uuid)
@@ -742,6 +896,155 @@ static void reconnect_device_profiles(struct spa_bt_monitor *monitor)
 	}
 }
 
+static struct spa_bt_remote_endpoint *device_remote_endpoint_find(struct spa_bt_device *device, const char *path)
+{
+	struct spa_bt_remote_endpoint *ep;
+	spa_list_for_each(ep, &device->remote_endpoint_list, device_link)
+		if (strcmp(ep->path, path) == 0)
+			return ep;
+	return NULL;
+}
+
+static struct spa_bt_remote_endpoint *remote_endpoint_find(struct spa_bt_monitor *monitor, const char *path)
+{
+	struct spa_bt_remote_endpoint *ep;
+	spa_list_for_each(ep, &monitor->remote_endpoint_list, link)
+		if (strcmp(ep->path, path) == 0)
+			return ep;
+	return NULL;
+}
+
+static int remote_endpoint_update_props(struct spa_bt_remote_endpoint *remote_endpoint,
+				DBusMessageIter *props_iter,
+				DBusMessageIter *invalidated_iter)
+{
+	struct spa_bt_monitor *monitor = remote_endpoint->monitor;
+
+	while (dbus_message_iter_get_arg_type(props_iter) != DBUS_TYPE_INVALID) {
+		DBusMessageIter it[2];
+		const char *key;
+		int type;
+
+		dbus_message_iter_recurse(props_iter, &it[0]);
+		dbus_message_iter_get_basic(&it[0], &key);
+		dbus_message_iter_next(&it[0]);
+		dbus_message_iter_recurse(&it[0], &it[1]);
+
+		type = dbus_message_iter_get_arg_type(&it[1]);
+
+		if (type == DBUS_TYPE_STRING || type == DBUS_TYPE_OBJECT_PATH) {
+			const char *value;
+
+			dbus_message_iter_get_basic(&it[1], &value);
+
+			spa_log_debug(monitor->log, "remote_endpoint %p: %s=%s", remote_endpoint, key, value);
+
+			if (strcmp(key, "UUID") == 0) {
+				free(remote_endpoint->uuid);
+				remote_endpoint->uuid = strdup(value);
+			}
+			else if (strcmp(key, "Device") == 0) {
+				struct spa_bt_device *device;
+				device = spa_bt_device_find(monitor, value);
+				spa_log_debug(monitor->log, "remote_endpoint %p: device -> %p", remote_endpoint, device);
+
+				if (remote_endpoint->device != device) {
+					if (remote_endpoint->device != NULL)
+						spa_list_remove(&remote_endpoint->device_link);
+					remote_endpoint->device = device;
+					if (device != NULL)
+						spa_list_append(&device->remote_endpoint_list, &remote_endpoint->device_link);
+				}
+			}
+		}
+		else if (type == DBUS_TYPE_BOOLEAN) {
+			int value;
+
+			dbus_message_iter_get_basic(&it[1], &value);
+
+			spa_log_debug(monitor->log, "remote_endpoint %p: %s=%d", remote_endpoint, key, value);
+
+			if (strcmp(key, "DelayReporting") == 0) {
+				remote_endpoint->delay_reporting = value;
+			}
+		}
+		else if (type == DBUS_TYPE_BYTE) {
+			uint8_t value;
+
+			dbus_message_iter_get_basic(&it[1], &value);
+
+			spa_log_debug(monitor->log, "remote_endpoint %p: %s=%02x", remote_endpoint, key, value);
+
+			if (strcmp(key, "Codec") == 0) {
+				remote_endpoint->codec = value;
+			}
+		}
+		else if (strcmp(key, "Capabilities") == 0) {
+			DBusMessageIter iter;
+			uint8_t *value;
+			int i, len;
+
+			if (!check_iter_signature(&it[1], "ay"))
+				goto next;
+
+			dbus_message_iter_recurse(&it[1], &iter);
+			dbus_message_iter_get_fixed_array(&iter, &value, &len);
+
+			spa_log_debug(monitor->log, "remote_endpoint %p: %s=%d", remote_endpoint, key, len);
+			for (i = 0; i < len; i++)
+				spa_log_debug(monitor->log, "  %d: %02x", i, value[i]);
+
+			free(remote_endpoint->capabilities);
+			remote_endpoint->capabilities_len = 0;
+
+			remote_endpoint->capabilities = malloc(len);
+			if (remote_endpoint->capabilities) {
+				memcpy(remote_endpoint->capabilities, value, len);
+				remote_endpoint->capabilities_len = len;
+			}
+		}
+		else
+			spa_log_debug(monitor->log, "remote_endpoint %p: unhandled key %s", remote_endpoint, key);
+
+	      next:
+		dbus_message_iter_next(props_iter);
+	}
+	return 0;
+}
+
+static struct spa_bt_remote_endpoint *remote_endpoint_create(struct spa_bt_monitor *monitor, const char *path)
+{
+	struct spa_bt_remote_endpoint *ep;
+
+	ep = calloc(1, sizeof(struct spa_bt_remote_endpoint));
+	if (ep == NULL)
+		return NULL;
+
+	ep->monitor = monitor;
+	ep->path = strdup(path);
+
+	spa_list_prepend(&monitor->remote_endpoint_list, &ep->link);
+
+	return ep;
+}
+
+static void remote_endpoint_free(struct spa_bt_remote_endpoint *remote_endpoint)
+{
+	struct spa_bt_monitor *monitor = remote_endpoint->monitor;
+
+	spa_log_debug(monitor->log, "remote endpoint %p: free %s",
+	              remote_endpoint, remote_endpoint->path);
+
+	if (remote_endpoint->device)
+		spa_list_remove(&remote_endpoint->device_link);
+
+	spa_list_remove(&remote_endpoint->link);
+	free(remote_endpoint->path);
+	free(remote_endpoint->uuid);
+	free(remote_endpoint->capabilities);
+	free(remote_endpoint);
+}
+
 struct spa_bt_transport *spa_bt_transport_find(struct spa_bt_monitor *monitor, const char *path)
 {
 	struct spa_bt_transport *t;
@@ -800,6 +1103,8 @@ static void transport_set_state(struct spa_bt_transport *transport, enum spa_bt_
 void spa_bt_transport_free(struct spa_bt_transport *transport)
 {
 	struct spa_bt_monitor *monitor = transport->monitor;
+	struct spa_bt_device *device = transport->device;
+	uint32_t prev_connected = 0;
 
 	spa_log_debug(monitor->log, "transport %p: free %s", transport, transport->path);
 
@@ -824,11 +1129,15 @@ void spa_bt_transport_free(struct spa_bt_transport *transport)
 
 	spa_list_remove(&transport->link);
 	if (transport->device) {
+		prev_connected = transport->device->connected_profiles;
 		transport->device->connected_profiles &= ~transport->profile;
 		spa_list_remove(&transport->device_link);
 	}
 	free(transport->path);
 	free(transport);
+
+	if (device && device->connected_profiles != prev_connected)
+		spa_bt_device_emit_profiles_changed(device, device->profiles, prev_connected);
 }
 
 int spa_bt_transport_acquire(struct spa_bt_transport *transport, bool optional)
@@ -995,7 +1304,7 @@ static int transport_update_props(struct spa_bt_transport *transport,
 			}
 		}
 		else if (strcmp(key, "Codec") == 0) {
-			int8_t value;
+			uint8_t value;
 
 			if (type != DBUS_TYPE_BYTE)
 				goto next;
@@ -1153,6 +1462,371 @@ static const struct spa_bt_transport_implementation transport_impl = {
 	.release = transport_release,
 };
 
+static void append_basic_array_variant_dict_entry(DBusMessageIter *dict, int key_type_int, void* key, const char* variant_type_str, const char* array_type_str, int array_type_int, void* data, int data_size);
+
+static void a2dp_codec_switch_reply(DBusPendingCall *pending, void *userdata);
+
+static int a2dp_codec_switch_cmp(const void *a, const void *b);
+
+static struct spa_bt_a2dp_codec_switch *a2dp_codec_switch_cmp_sw;  /* global for qsort */
+
+static void a2dp_codec_switch_free(struct spa_bt_a2dp_codec_switch *sw)
+{
+	char **p;
+
+	if (sw->pending != NULL) {
+		dbus_pending_call_cancel(sw->pending);
+		dbus_pending_call_unref(sw->pending);
+	}
+
+	if (sw->device != NULL)
+		spa_list_remove(&sw->device_link);
+
+	if (sw->paths != NULL)
+		for (p = sw->paths; *p != NULL; ++p)
+			free(*p);
+
+	free(sw->paths);
+	free(sw->codecs);
+	free(sw);
+}
+
+static void a2dp_codec_switch_next(struct spa_bt_a2dp_codec_switch *sw)
+{
+	spa_assert(*sw->codec_iter != NULL && *sw->path_iter != NULL);
+
+	++sw->path_iter;
+	if (*sw->path_iter == NULL) {
+		++sw->codec_iter;
+		sw->path_iter = sw->paths;
+	}
+}
+
+static bool a2dp_codec_switch_process_current(struct spa_bt_a2dp_codec_switch *sw)
+{
+	struct spa_bt_remote_endpoint *ep;
+	const struct a2dp_codec *codec;
+	uint8_t config[A2DP_MAX_CAPS_SIZE];
+	char *local_endpoint_base;
+	char *local_endpoint = NULL;
+	int res, config_size;
+	dbus_bool_t dbus_ret;
+	const char *str;
+	DBusMessage *m;
+	DBusMessageIter iter, d;
+	int i;
+
+	/* Try setting configuration for current codec on current endpoint in list */
+
+	codec = *sw->codec_iter;
+
+	spa_log_debug(sw->device->monitor->log, NAME": a2dp codec switch %p: consider codec %s for remote endpoint %s",
+	              sw, (*sw->codec_iter)->name, *sw->path_iter);
+
+	ep = device_remote_endpoint_find(sw->device, *sw->path_iter);
+
+	if (ep == NULL || ep->capabilities == NULL || ep->uuid == NULL) {
+		spa_log_debug(sw->device->monitor->log, NAME": a2dp codec switch %p: endpoint %s not valid, try next",
+		              sw, *sw->path_iter);
+		goto next;
+	}
+
+	/* Setup and check compatible configuration */
+	if (ep->codec != codec->codec_id) {
+		spa_log_debug(sw->device->monitor->log, NAME": a2dp codec switch %p: different codec, try next", sw);
+		goto next;
+	}
+
+	if (!(sw->profile & spa_bt_profile_from_uuid(ep->uuid))) {
+		spa_log_debug(sw->device->monitor->log, NAME": a2dp codec switch %p: wrong uuid (%s) for profile, try next",
+		              sw, ep->uuid);
+		goto next;
+	}
+
+	if (sw->profile & SPA_BT_PROFILE_A2DP_SINK) {
+		local_endpoint_base = A2DP_SOURCE_ENDPOINT;
+	} else if (sw->profile & SPA_BT_PROFILE_A2DP_SOURCE) {
+		local_endpoint_base = A2DP_SINK_ENDPOINT;
+	} else {
+		spa_log_debug(sw->device->monitor->log, NAME": a2dp codec switch %p: bad profile (%d), try next",
+		              sw, sw->profile);
+		goto next;
+	}
+
+	if (a2dp_codec_to_endpoint(codec, local_endpoint_base, &local_endpoint) < 0) {
+		spa_log_debug(sw->device->monitor->log, NAME": a2dp codec switch %p: no endpoint for codec %s, try next",
+		              sw, codec->name);
+		goto next;
+	}
+
+	res = codec->select_config(codec, 0, ep->capabilities, ep->capabilities_len, NULL, config);
+	if (res < 0) {
+		spa_log_debug(sw->device->monitor->log, NAME": a2dp codec switch %p: incompatible capabilities (%d), try next",
+		              sw, res);
+		goto next;
+	}
+	config_size = res;
+
+	spa_log_debug(sw->device->monitor->log, NAME": a2dp codec switch %p: configuration %d", sw, config_size);
+	for (i = 0; i < config_size; i++)
+		spa_log_debug(sw->device->monitor->log, NAME": a2dp codec switch %p:     %d: %02x", sw, i, config[i]);
+
+	/* org.bluez.MediaEndpoint1.SetConfiguration on remote endpoint */
+	m = dbus_message_new_method_call(BLUEZ_SERVICE, ep->path, BLUEZ_MEDIA_ENDPOINT_INTERFACE, "SetConfiguration");
+	if (m == NULL) {
+		spa_log_debug(sw->device->monitor->log, NAME": a2dp codec switch %p: dbus allocation failure, try next", sw);
+		goto next;
+	}
+
+	spa_log_info(sw->device->monitor->log, NAME": a2dp codec switch %p: trying codec %s for endpoint %s, local endpoint %s",
+	             sw, codec->name, ep->path, local_endpoint);
+
+	dbus_message_iter_init_append(m, &iter);
+	dbus_message_iter_append_basic(&iter, DBUS_TYPE_OBJECT_PATH, &local_endpoint);
+	dbus_message_iter_open_container(&iter, DBUS_TYPE_ARRAY, "{sv}", &d);
+	str = "Capabilities";
+	append_basic_array_variant_dict_entry(&d, DBUS_TYPE_STRING, &str, "ay", "y", DBUS_TYPE_BYTE, config, config_size);
+	dbus_message_iter_close_container(&iter, &d);
+
+	spa_assert(sw->pending == NULL);
+	dbus_ret = dbus_connection_send_with_reply(sw->device->monitor->conn, m, &sw->pending, -1);
+
+	if (!dbus_ret || sw->pending == NULL) {
+		spa_log_error(sw->device->monitor->log, NAME": a2dp codec switch %p: dbus call failure, try next", sw);
+		dbus_message_unref(m);
+		goto next;
+	}
+
+	dbus_ret = dbus_pending_call_set_notify(sw->pending, a2dp_codec_switch_reply, sw, NULL);
+	dbus_message_unref(m);
+
+	if (!dbus_ret) {
+		spa_log_error(sw->device->monitor->log, NAME": a2dp codec switch %p: dbus set notify failure", sw);
+		goto next;
+	}
+
+	free(local_endpoint);
+	return true;
+
+next:
+	free(local_endpoint);
+	return false;
+}
+
+static void a2dp_codec_switch_process(struct spa_bt_a2dp_codec_switch *sw)
+{
+	while (*sw->codec_iter != NULL && *sw->path_iter != NULL) {
+		if (sw->path_iter == sw->paths && (*sw->codec_iter)->caps_preference_cmp) {
+			/* Sort endpoints according to codec preference, when at a new codec. */
+			a2dp_codec_switch_cmp_sw = sw;
+			qsort(sw->paths, sw->num_paths, sizeof(char *), a2dp_codec_switch_cmp);
+		}
+
+		if (a2dp_codec_switch_process_current(sw)) {
+			/* Wait for dbus reply */
+			return;
+		}
+
+		a2dp_codec_switch_next(sw);
+	};
+
+	/* Didn't find any suitable endpoint. Report failure. */
+	spa_log_info(sw->device->monitor->log, NAME": a2dp codec switch %p: failed to get an endpoint", sw);
+	sw->device->active_codec_switch = NULL;
+	spa_bt_device_emit_codec_switched(sw->device, -ENODEV);
+	spa_bt_device_check_profiles(sw->device, false);
+	a2dp_codec_switch_free(sw);
+}
+
+static void a2dp_codec_switch_reply(DBusPendingCall *pending, void *user_data)
+{
+	struct spa_bt_a2dp_codec_switch *sw = user_data;
+	struct spa_bt_device *device = sw->device;
+	DBusMessage *r;
+
+	r = dbus_pending_call_steal_reply(pending);
+
+	spa_assert(sw->pending == pending);
+	dbus_pending_call_unref(pending);
+	sw->pending = NULL;
+
+	if (sw->device->active_codec_switch != sw) {
+		/* This codec switch has been canceled. Switch to the new one. */
+		spa_log_debug(sw->device->monitor->log, NAME": a2dp codec switch %p: canceled, go to new switch", sw);
+		a2dp_codec_switch_free(sw);
+
+		if (r != NULL)
+			dbus_message_unref(r);
+
+		spa_assert(device->active_codec_switch != NULL);
+		a2dp_codec_switch_process(device->active_codec_switch);
+		return;
+	}
+
+	if (r == NULL) {
+		spa_log_error(sw->device->monitor->log,
+		              NAME": a2dp codec switch %p: empty reply from dbus, trying next",
+		              sw);
+		goto next;
+	}
+
+	if (dbus_message_get_type(r) == DBUS_MESSAGE_TYPE_ERROR) {
+		spa_log_debug(sw->device->monitor->log,
+		              NAME": a2dp codec switch %p: failed (%s), trying next",
+		              sw, dbus_message_get_error_name(r));
+		dbus_message_unref(r);
+		goto next;
+	}
+
+	dbus_message_unref(r);
+
+	/* Success */
+	spa_log_info(sw->device->monitor->log, NAME": a2dp codec switch %p: success", sw);
+	device->active_codec_switch = NULL;
+	spa_bt_device_emit_codec_switched(sw->device, 0);
+	spa_bt_device_check_profiles(sw->device, false);
+	a2dp_codec_switch_free(sw);
+	return;
+
+next:
+	a2dp_codec_switch_next(sw);
+	a2dp_codec_switch_process(sw);
+	return;
+}
+
+static int a2dp_codec_switch_cmp(const void *a, const void *b)
+{
+	struct spa_bt_a2dp_codec_switch *sw = a2dp_codec_switch_cmp_sw;
+	const struct a2dp_codec *codec = *sw->codec_iter;
+	const char *path1 = a, *path2 = b;
+	struct spa_bt_remote_endpoint *ep1, *ep2;
+
+	ep1 = device_remote_endpoint_find(sw->device, path1);
+	ep2 = device_remote_endpoint_find(sw->device, path2);
+
+	if (ep1 != NULL && (ep1->uuid == NULL || ep1->codec != codec->codec_id || ep1->capabilities == NULL))
+		ep1 = NULL;
+	if (ep2 != NULL && (ep2->uuid == NULL || ep2->codec != codec->codec_id || ep2->capabilities == NULL))
+		ep2 = NULL;
+
+	if (ep1 == NULL && ep2 == NULL)
+		return 0;
+	else if (ep1 == NULL)
+		return 1;
+	else if (ep2 == NULL)
+		return -1;
+
+	return codec->caps_preference_cmp(codec, ep1->capabilities, ep1->capabilities_len,
+			ep2->capabilities, ep2->capabilities_len);
+}
+
+/* Ensure there's a transport for at least one of the listed codecs */
+int spa_bt_device_ensure_a2dp_codec(struct spa_bt_device *device, const struct a2dp_codec **codecs)
+{
+	struct spa_bt_a2dp_codec_switch *sw;
+	struct spa_bt_remote_endpoint *ep;
+	struct spa_bt_transport *t;
+	const struct a2dp_codec *preferred_codec = NULL;
+	size_t i, j, num_codecs, num_eps;
+
+	if (!device->adapter->application_registered) {
+		/* Codec switching not supported */
+		return -ENOTSUP;
+	}
+
+	for (i = 0; codecs[i] != NULL; ++i) {
+		if (spa_bt_device_supports_a2dp_codec(device, codecs[i])) {
+			preferred_codec = codecs[i];
+			break;
+		}
+	}
+
+	/* Check if we already have an enabled transport for the most preferred codec.
+	 * However, if there already was a codec switch running, these transports may
+	 * disapper soon. In that case, we have to do the full thing.
+	 */
+	if (device->active_codec_switch == NULL && preferred_codec != NULL) {
+		spa_list_for_each(t, &device->transport_list, device_link) {
+			if (t->a2dp_codec != preferred_codec || !t->enabled)
+				continue;
+
+			if ((device->connected_profiles & t->profile) != t->profile)
+				continue;
+
+			spa_bt_device_emit_codec_switched(device, 0);
+			return 0;
+		}
+	}
+
+	/* Setup and start iteration */
+
+	sw = calloc(1, sizeof(struct spa_bt_a2dp_codec_switch));
+	if (sw == NULL)
+		return -ENOMEM;
+
+	num_eps = 0;
+	spa_list_for_each(ep, &device->remote_endpoint_list, device_link)
+		++num_eps;
+
+	num_codecs = 0;
+	while (codecs[num_codecs] != NULL)
+		++num_codecs;
+
+	sw->codecs = calloc(num_codecs + 1, sizeof(const struct a2dp_codec *));
+	sw->paths = calloc(num_eps + 1, sizeof(char *));
+	sw->num_paths = num_eps;
+
+	if (sw->codecs == NULL || sw->paths == NULL) {
+		a2dp_codec_switch_free(sw);
+		return -ENOMEM;
+	}
+
+	for (i = 0, j = 0; i < num_codecs; ++i) {
+		if (device_can_accept_a2dp_codec(device, codecs[i])) {
+			sw->codecs[j] = codecs[i];
+			++j;
+		}
+	}
+	sw->codecs[j] = NULL;
+
+	i = 0;
+	spa_list_for_each(ep, &device->remote_endpoint_list, device_link) {
+		sw->paths[i] = strdup(ep->path);
+		if (sw->paths[i] == NULL) {
+			a2dp_codec_switch_free(sw);
+			return -ENOMEM;
+		}
+		++i;
+	}
+	sw->paths[i] = NULL;
+
+	sw->codec_iter = sw->codecs;
+	sw->path_iter = sw->paths;
+
+	sw->profile = device->connected_profiles;
+
+	sw->device = device;
+	spa_list_append(&device->codec_switch_list, &sw->device_link);
+
+	if (device->active_codec_switch != NULL) {
+		/*
+		 * There's a codec switch already running.  BlueZ does not appear to allow
+		 * calling dbus_pending_call_cancel on an active request, so we have to
+		 * wait for the reply to arrive first, and only then start processing this
+		 * request.
+		 */
+		spa_log_debug(sw->device->monitor->log,
+		             NAME": a2dp codec switch: already in progress, canceling previous");
+		spa_assert(device->active_codec_switch->pending != NULL);
+		device->active_codec_switch = sw;
+	} else {
+		device->active_codec_switch = sw;
+		a2dp_codec_switch_process(sw);
+	}
+
+	return 0;
+}
+
 static DBusHandlerResult endpoint_set_configuration(DBusConnection *conn,
 		const char *path, DBusMessage *m, void *userdata)
 {
@@ -1220,6 +1894,16 @@ static DBusHandlerResult endpoint_set_configuration(DBusConnection *conn,
 	spa_log_info(monitor->log, "%p: %s validate conf channels:%d",
 			monitor, path, transport->n_channels);
 
+	/*
+	 * Per-device determination of which codecs are allowed needs to be done also
+	 * here. The A2DP codec switching process does this filtering, but before that
+	 * BlueZ may autoconnect to any endpoint with compatible caps.  In case it got it
+	 * wrong, mark the transport disabled, and we'll switch to a better one later when
+	 * needed. (We don't want to fail the connection, because we want to keep the
+	 * profile connected.)
+	 */
+	transport->enabled = device_can_accept_a2dp_codec(transport->device, codec);
+
 	spa_bt_device_connect_profile(transport->device, transport->profile);
 
 	if ((r = dbus_message_new_method_return(m)) == NULL)
@@ -1475,7 +2159,10 @@ static int adapter_register_endpoints(struct spa_bt_adapter *a)
 	for (i = 0; a2dp_codecs[i]; i++) {
 		const struct a2dp_codec *codec = a2dp_codecs[i];
 
-		if (codec->codec_id != A2DP_CODEC_SBC)
+		if (!is_a2dp_codec_enabled(monitor, codec))
+			continue;
+
+		if (!(codec->codec_id == A2DP_CODEC_SBC && strcmp(codec->name, "sbc") == 0))
 			continue;
 
 		if ((err = bluez_register_endpoint(monitor, a->path,
@@ -1579,6 +2266,9 @@ static DBusHandlerResult object_manager_handler(DBusConnection *c, DBusMessage *
 			int caps_size, ret;
 			uint16_t codec_id = codec->codec_id;
 
+			if (!is_a2dp_codec_enabled(monitor, codec))
+				continue;
+
 			caps_size = codec->fill_caps(codec, 0, caps);
 			if (caps_size < 0)
 				continue;
@@ -1666,6 +2356,10 @@ static int register_media_application(struct spa_bt_monitor * monitor)
 
 	for (int i = 0; a2dp_codecs[i]; i++) {
 		const struct a2dp_codec *codec = a2dp_codecs[i];
+
+		if (!is_a2dp_codec_enabled(monitor, codec))
+			continue;
+
 		register_a2dp_endpoint(monitor, codec, A2DP_SOURCE_ENDPOINT);
 		register_a2dp_endpoint(monitor, codec, A2DP_SINK_ENDPOINT);
 	}
@@ -1682,6 +2376,9 @@ static void unregister_media_application(struct spa_bt_monitor * monitor)
 	for (int i = 0; a2dp_codecs[i]; i++) {
 		const struct a2dp_codec *codec = a2dp_codecs[i];
 
+		if (!is_a2dp_codec_enabled(monitor, codec))
+			continue;
+
 		ret = a2dp_codec_to_endpoint(codec, A2DP_SOURCE_ENDPOINT, &object_path);
 		if (ret == 0) {
 			dbus_connection_unregister_object_path(monitor->conn, object_path);
@@ -1766,6 +2463,20 @@ static void interface_added(struct spa_bt_monitor *monitor,
 		}
 		device_update_props(d, props_iter, NULL);
 	}
+	else if (strcmp(interface_name, BLUEZ_MEDIA_ENDPOINT_INTERFACE) == 0) {
+		struct spa_bt_remote_endpoint *ep;
+
+		ep = remote_endpoint_find(monitor, object_path);
+		if (ep == NULL) {
+			ep = remote_endpoint_create(monitor, object_path);
+			if (ep == NULL) {
+				spa_log_warn(monitor->log, "can't create Bluetooth remote endpoint %s: %m",
+				             object_path);
+				return;
+			}
+		}
+		remote_endpoint_update_props(ep, props_iter, NULL);
+	}
 }
 
 static void interfaces_added(struct spa_bt_monitor *monitor, DBusMessageIter *arg_iter)
@@ -1819,6 +2530,11 @@ static void interfaces_removed(struct spa_bt_monitor *monitor, DBusMessageIter *
 			a = adapter_find(monitor, object_path);
 			if (a != NULL)
 				adapter_free(a);
+		} else if (strcmp(interface_name, BLUEZ_MEDIA_ENDPOINT_INTERFACE) == 0) {
+			struct spa_bt_remote_endpoint *ep;
+			ep = remote_endpoint_find(monitor, object_path);
+			if (ep != NULL)
+				remote_endpoint_free(ep);
 		}
 
 		dbus_message_iter_next(&it);
@@ -1910,6 +2626,7 @@ static DBusHandlerResult filter_cb(DBusConnection *bus, DBusMessage *m, void *us
 			if (old_owner && *old_owner) {
 				struct spa_bt_adapter *a;
 				struct spa_bt_device *d;
+				struct spa_bt_remote_endpoint *ep;
 				struct spa_bt_transport *t;
 
 				spa_log_debug(monitor->log, "Bluetooth daemon disappeared");
@@ -1917,6 +2634,8 @@ static DBusHandlerResult filter_cb(DBusConnection *bus, DBusMessage *m, void *us
 
 				spa_list_consume(t, &monitor->transport_list, link)
 					spa_bt_transport_free(t);
+				spa_list_consume(ep, &monitor->remote_endpoint_list, link)
+					remote_endpoint_free(ep);
 				spa_list_consume(d, &monitor->device_list, link)
 					device_free(d);
 				spa_list_consume(a, &monitor->adapter_list, link)
@@ -2000,6 +2719,19 @@ static DBusHandlerResult filter_cb(DBusConnection *bus, DBusMessage *m, void *us
 
 			device_update_props(d, &it[1], NULL);
 		}
+		else if (strcmp(iface, BLUEZ_MEDIA_ENDPOINT_INTERFACE) == 0) {
+			struct spa_bt_remote_endpoint *ep;
+
+			ep = remote_endpoint_find(monitor, path);
+			if (ep == NULL) {
+				spa_log_debug(monitor->log,
+						"Properties changed in unknown remote endpoint %s", path);
+				goto finish;
+			}
+			spa_log_debug(monitor->log, "Properties changed in remote endpoint %s", path);
+
+			remote_endpoint_update_props(ep, &it[1], NULL);
+		}
 		else if (strcmp(iface, BLUEZ_MEDIA_TRANSPORT_INTERFACE) == 0) {
 			struct spa_bt_transport *transport;
 
@@ -2054,6 +2786,10 @@ static void add_filters(struct spa_bt_monitor *this)
 			"type='signal',sender='" BLUEZ_SERVICE "',"
 			"interface='org.freedesktop.DBus.Properties',member='PropertiesChanged',"
 			"arg0='" BLUEZ_DEVICE_INTERFACE "'", &err);
+	dbus_bus_add_match(this->conn,
+			"type='signal',sender='" BLUEZ_SERVICE "',"
+			"interface='org.freedesktop.DBus.Properties',member='PropertiesChanged',"
+			"arg0='" BLUEZ_MEDIA_ENDPOINT_INTERFACE "'", &err);
 	dbus_bus_add_match(this->conn,
 			"type='signal',sender='" BLUEZ_SERVICE "',"
 			"interface='org.freedesktop.DBus.Properties',member='PropertiesChanged',"
@@ -2120,6 +2856,7 @@ static int impl_clear(struct spa_handle *handle)
 	struct spa_bt_monitor *monitor;
 	struct spa_bt_adapter *a;
 	struct spa_bt_device *d;
+	struct spa_bt_remote_endpoint *ep;
 	struct spa_bt_transport *t;
 
 	monitor = (struct spa_bt_monitor *) handle;
@@ -2128,6 +2865,8 @@ static int impl_clear(struct spa_handle *handle)
 
 	spa_list_consume(t, &monitor->transport_list, link)
 		spa_bt_transport_free(t);
+	spa_list_consume(ep, &monitor->remote_endpoint_list, link)
+		remote_endpoint_free(ep);
 	spa_list_consume(d, &monitor->device_list, link)
 		device_free(d);
 	spa_list_consume(a, &monitor->adapter_list, link)
@@ -2148,6 +2887,9 @@ static int impl_clear(struct spa_handle *handle)
 		monitor->backend_hsphfpd = NULL;
 	}
 
+	free((void*)monitor->enabled_codecs.items);
+	spa_zero(monitor->enabled_codecs);
+
 	return 0;
 }
 
@@ -2158,6 +2900,82 @@ impl_get_size(const struct spa_handle_factory *factory,
 	return sizeof(struct spa_bt_monitor);
 }
 
+static int parse_codec_array(struct spa_bt_monitor *this, const struct spa_dict *info)
+{
+	const char *str;
+	struct spa_dict_item *codecs;
+	struct spa_json it, it_array;
+	char codec_name[256];
+	size_t num_codecs;
+	int i;
+
+	/* Parse bluez5.codecs property to a dict of enabled codecs */
+
+	num_codecs = 0;
+	while (a2dp_codecs[num_codecs])
+		++num_codecs;
+
+	codecs = calloc(num_codecs, sizeof(struct spa_dict_item));
+	if (codecs == NULL)
+		return -ENOMEM;
+
+	if ((str = spa_dict_lookup(info, "bluez5.codecs")) == NULL)
+		goto fallback;
+
+	spa_json_init(&it, str, strlen(str));
+
+	if (spa_json_enter_array(&it, &it_array) <= 0) {
+		spa_log_error(this->log, NAME": property bluez5.codecs '%s' is not an array", str);
+		goto fallback;
+	}
+
+	this->enabled_codecs = SPA_DICT_INIT(codecs, 0);
+
+	while (spa_json_get_string(&it_array, codec_name, sizeof(codec_name)) > 0) {
+		int i;
+
+		for (i = 0; a2dp_codecs[i]; ++i) {
+			const struct a2dp_codec *codec = a2dp_codecs[i];
+
+			if (strcmp(codec->name, codec_name) != 0)
+				continue;
+
+			if (spa_dict_lookup_item(&this->enabled_codecs, codec->name) != NULL)
+				continue;
+
+			spa_log_debug(this->log, NAME": enabling codec %s", codec->name);
+
+			spa_assert(this->enabled_codecs.n_items < num_codecs);
+
+			codecs[this->enabled_codecs.n_items].key = codec->name;
+			codecs[this->enabled_codecs.n_items].value = "true";
+			++this->enabled_codecs.n_items;
+
+			break;
+		}
+	}
+
+	spa_dict_qsort(&this->enabled_codecs);
+
+	for (i = 0; a2dp_codecs[i]; ++i) {
+		const struct a2dp_codec *codec = a2dp_codecs[i];
+		if (!is_a2dp_codec_enabled(this, codec))
+			spa_log_debug(this->log, NAME": disabling codec %s", codec->name);
+	}
+	return 0;
+
+fallback:
+	for (i = 0; a2dp_codecs[i]; ++i) {
+		const struct a2dp_codec *codec = a2dp_codecs[i];
+		spa_log_debug(this->log, NAME": enabling codec %s", codec->name);
+		codecs[i].key = codec->name;
+		codecs[i].value = "true";
+	}
+	this->enabled_codecs = SPA_DICT_INIT(codecs, i);
+	spa_dict_qsort(&this->enabled_codecs);
+	return 0;
+}
+
 static int
 impl_init(const struct spa_handle_factory *factory,
 	  struct spa_handle *handle,
@@ -2166,6 +2984,7 @@ impl_init(const struct spa_handle_factory *factory,
 	  uint32_t n_support)
 {
 	struct spa_bt_monitor *this;
+	int res;
 
 	spa_return_val_if_fail(factory != NULL, -EINVAL);
 	spa_return_val_if_fail(handle != NULL, -EINVAL);
@@ -2201,8 +3020,12 @@ impl_init(const struct spa_handle_factory *factory,
 
 	spa_list_init(&this->adapter_list);
 	spa_list_init(&this->device_list);
+	spa_list_init(&this->remote_endpoint_list);
 	spa_list_init(&this->transport_list);
 
+	if ((res = parse_codec_array(this, info)) < 0)
+		return res;
+
 	register_media_application(this);
 
 	if (info) {
