@@ -297,6 +297,9 @@ int sm_media_session_load_state(struct sm_media_session *sess,
 int sm_media_session_save_state(struct sm_media_session *sess,
 		const char *name, const char *prefix, const struct pw_properties *props);
 
+int sm_media_session_match_rules(const char *rules, size_t size,
+		struct pw_properties *props);
+
 #ifdef __cplusplus
 }
 #endif
