@@ -575,6 +575,8 @@ static void registry_event_global_remove(void *object, uint32_t id)
 	if ((o = find_object(m, id)) == NULL)
 		return;
 
+	o->this.removing = true;
+
 	manager_emit_removed(m, &o->this);
 
 	object_destroy(o);
@@ -689,7 +691,7 @@ int pw_manager_set_metadata(struct pw_manager *manager,
 
 	if (type != NULL) {
 		va_start(args, format);
-		vsnprintf(buf, sizeof(buf)-1, format, args);
+		vsnprintf(buf, sizeof(buf), format, args);
 		va_end(args);
 		value = buf;
 	} else {
